package at.htl.payyours

import android.content.Context
import android.content.Intent
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.widget.EditText
import android.widget.TextView
import kotlinx.android.synthetic.main.activity_main.*

class MainActivity : AppCompatActivity() {

    companion object {
        val TAG = MainActivity::class.java.simpleName
        val PAYMENT_KEY = "Payment"
    }
    private lateinit var payment: Payment

    override fun onCreate(savedInstanceState: Bundle?) {
        Log.d(TAG, "onCreate()")
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        buttonCourtsPlus.setOnClickListener{ add(editCourts, 1) }
        buttonCourtsMinus.setOnClickListener{ add(editCourts, -1) }
        buttonPlayersPlus.setOnClickListener{ add(editPlayers, 1) }
        buttonPlayersMinus.setOnClickListener{ add(editPlayers, -1) }

        editCourts.setOnFocusChangeListener { view, hasFocus -> onFocusChanged(view as EditText, hasFocus) }
        editPlayers.setOnFocusChangeListener { view, hasFocus -> onFocusChanged(view as EditText, hasFocus) }


        payment = if (savedInstanceState == null) {
            SettingsActivity.defaultPayment(this.applicationContext.getSharedPreferences(
                SettingsActivity.PREFERENCE_FILENAME, Context.MODE_PRIVATE))
        }
        else{
            savedInstanceState.get(PAYMENT_KEY) as Payment
        }
        editPlayers.setText("${payment.players}")
        editCourts.setText("${payment.courts}")
        updatePayment()
    }

    override fun onSaveInstanceState(outState: Bundle?) {
        outState?.putSerializable(PAYMENT_KEY, payment)
        super.onSaveInstanceState(outState)
    }

    override fun onRestoreInstanceState(savedInstanceState: Bundle?) {
        Log.d(TAG, "onRestoreInstanceState()")
        super.onRestoreInstanceState(savedInstanceState)
    }

    override fun onResume() {
        Log.d(TAG, "onResume() {\n()")
        super.onResume()
        payment.pricePerUnit = readPricePerUnit()
        updatePayment()
    }

    private fun add(t: TextView, value: Int) {
        val v = t.text.toString().toInt() + value
        t.setText("$v")
        updatePayment()
    }

    private fun onFocusChanged(editText: EditText, hasFocus: Boolean) {
        Log.d(TAG, "In onFocusChanged for ${editText.id}")
        if (hasFocus) {
            editText.text.clear()
        } else {
            updatePayment()
        }
    }

    private fun updatePayment() {
        payment.players = editPlayers.text.toString().toInt()
        payment.courts = editCourts.text.toString().toInt()
        textViewPayAmount.setText("${payment.amount}")
        buttonPlayersMinus.isEnabled = payment.players > 1
        buttonCourtsMinus.isEnabled = payment.courts > 1
        Log.d(TAG, "Payment updated: $payment")
    }

    private fun readPricePerUnit() : Double{
        val preferences = this.applicationContext.getSharedPreferences(
            SettingsActivity.PREFERENCE_FILENAME, Context.MODE_PRIVATE)
        return preferences.getFloat(SettingsActivity.PRICE_PER_UNIT_KEY, payment.pricePerUnit.toFloat()).toDouble()
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem?) =
        when (item?.itemId) {
            R.id.menu_item_quit -> {
                finish()
                true
            }
            R.id.menu_item_settings -> {
                Log.d("MainActivity", "Settings menu called")
                startActivity(Intent(this, SettingsActivity::class.java))
                true
            }
            else -> super.onOptionsItemSelected(item)
        }

}
