package at.htl.payyours

import android.content.Context
import android.content.Intent
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.widget.EditText
import android.widget.TextView
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import kotlinx.android.synthetic.main.activity_main.*


class MainActivity : AppCompatActivity(), LifecycleOwner {

    companion object {
        val TAG = MainActivity::class.java.simpleName
        val PAYMENT_KEY = "Payment"
    }

    private lateinit var model: MainActivityViewModel
    private val payment: Payment get(){
        return model.payment.value!!;
    }

    override fun onStart() {
        Log.d(TAG, "onStart(): $payment")
        super.onStart()
    }

    override fun onStop() {
        Log.d(TAG, "onStop(): $payment")
        super.onStop()
    }

    override fun onPause() {
        Log.d(TAG, "onPause(): $payment")
        super.onPause()
    }

    override fun onResume() {
        Log.d(TAG, "onResume(): $payment")
        payment.pricePerUnit = readPricePerUnit()
        updatePayment()
        super.onResume()
    }

    override fun onDestroy() {
        Log.d(TAG, "onDestroy(): $payment")
        super.onDestroy()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        Log.d(TAG, "onCreate()")
        super.onCreate(savedInstanceState)
        lifecycle.addObserver(MainActivityObserver())
        setContentView(R.layout.activity_main)

        
        buttonCourtsPlus.setOnClickListener{ add(editCourts, 1) }
        buttonCourtsMinus.setOnClickListener{ add(editCourts, -1) }
        buttonPlayersPlus.setOnClickListener{ add(editPlayers, 1) }
        buttonPlayersMinus.setOnClickListener{ add(editPlayers, -1) }

        editCourts.setOnFocusChangeListener { view, hasFocus -> onFocusChanged(view as EditText, hasFocus) }
        editPlayers.setOnFocusChangeListener { view, hasFocus -> onFocusChanged(view as EditText, hasFocus) }

        model = ViewModelProviders.of(this).get(MainActivityViewModel::class.java)
        model.payment.observe(this, Observer<Payment>{
                p -> displayPayment()
        })

        if (savedInstanceState == null) {
            model.payment.value = SettingsActivity.defaultPayment(this.applicationContext.getSharedPreferences(
                SettingsActivity.PREFERENCE_FILENAME, Context.MODE_PRIVATE))
        }
	    Log.d(TAG, "onCreate(): $payment")
        //displayPayment()
    }

    private fun add(t: TextView, value: Int) {
        val v = t.text.toString().toInt() + value
        t.setText("$v")
        updatePayment()
    }

    private fun onFocusChanged(editText: EditText, hasFocus: Boolean) {
        Log.d(TAG, "In onFocusChanged for ${editText.id}")
        if (hasFocus) {
            editText.text.clear()
        } else {
            updatePayment()
        }
    }

    private fun updatePayment() {
        payment.players = editPlayers.text.toString().toInt()
        payment.courts = editCourts.text.toString().toInt()
        model.payment.value = this.payment
        Log.d(TAG, "Payment updated: ${payment})")
    }

    private fun displayPayment(){
        editPlayers.setText("${payment.players}")
        editCourts.setText("${payment.courts}")
        textViewPayAmount.setText("${payment.amount}")
        Log.d(TAG, "Payment displayed: $payment")
    }

    private fun readPricePerUnit() : Double{
        val preferences = this.applicationContext.getSharedPreferences(
            SettingsActivity.PREFERENCE_FILENAME, Context.MODE_PRIVATE)
        return preferences.getFloat(SettingsActivity.PRICE_PER_UNIT_KEY, payment.pricePerUnit.toFloat()).toDouble()
    }
    
    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem?) =
        when (item?.itemId) {
            R.id.menu_item_quit -> {
                finish()
                true
            }
            R.id.menu_item_settings -> {
                Log.d("MainActivity", "Settings menu called")
                startActivity(Intent(this, SettingsActivity::class.java))
                true
            }
            else -> super.onOptionsItemSelected(item)
        }

}
