package at.htl.payyours

import android.app.Application
import android.content.Context
import android.util.Log
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations


class MainActivityViewModel(val app: Application) : AndroidViewModel(app){
    companion object {
        val TAG = MainActivityViewModel::class.java.simpleName
    }
    private val _payment = MutableLiveData<Payment>()
    val payment: LiveData<Payment>
        get() = _payment

    init{
        _payment.value =
            SettingsActivity.defaultPayment(app.getSharedPreferences(
                SettingsActivity.PREFERENCE_FILENAME, Context.MODE_PRIVATE))
    }

    val playersString = Transformations.map(_payment){p -> "" + p.players }
    val courtsString = Transformations.map(_payment) {p -> "" + p.courts }
    val amountString = Transformations.map(_payment) {p -> "" + p.amount}

    val decrementPlayersVisible = Transformations.map(_payment){p -> p.players > 1}
    val decrementCourtsVisible = Transformations.map(_payment){p -> p.courts > 1}

    fun onPlayersTextChanged(text: CharSequence ){
        Log.d(TAG, "Players text changed: " + text)
        try {
            val players = Integer.parseInt(text.toString())
            val p = _payment.value
            p?.let {
                if (it.players != players) {
                    it.players = players
                    _payment.value = it
                }
            }
        }
        catch(e: NumberFormatException){
            Log.e(TAG, "Players parse error: " + e.message)
        }
    }

    fun onCourtsTextChanged(text: CharSequence ){
        Log.d(TAG, "Courts text changed: " + text)
        try{
            val courts = Integer.parseInt(text.toString())
            val p = _payment.value
            p?.let{
                if (it.courts != courts){
                    it.courts = courts
                    _payment.value = it
                }
            }
        }
        catch(e: NumberFormatException){
            Log.e(TAG, "Courts parse error: " + e.message)
        }

    }

    fun onUpdatePlayers(value: Int){
        val p = _payment.value
        p?.let{
            it.players = it.players + value
            _payment.value = p
        }
    }

    fun onUpdateCourts(value: Int){
        val p = _payment.value
        p?.let{
            it.courts = it.courts + value
            _payment.value = p
        }
    }

    fun onUpdatePricePerUnit() {
        val preferences = app.getSharedPreferences(
            SettingsActivity.PREFERENCE_FILENAME, Context.MODE_PRIVATE)
                val p = _payment.value
        p?.let{
            val ppu =  preferences.getFloat(
                SettingsActivity.PRICE_PER_UNIT_KEY,
                SettingsActivity.DEFAULT_PRICE_PER_UNIT).toDouble()
            it.pricePerUnit = ppu
            _payment.value = it
        }
    }
}