package at.htl.payyours

import android.app.Activity
import android.content.Context
import android.content.SharedPreferences
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import kotlinx.android.synthetic.main.activity_settings.*
import java.text.DecimalFormat
import java.util.*

class SettingsActivity : AppCompatActivity() {

    companion object {
        val PREFERENCE_FILENAME = "PayYoursPreferences"
        val PRICE_PER_UNIT_KEY = "PRICE_PER_UNIT"
        val PLAYERS_KEY = "DEFAULT_PLAYERS"
        val COURTS_KEY = "DEFAULT_COURTS"
        val DEFAULT_PRICE_PER_UNIT = 10.0F

        fun defaultPayment(preferences: SharedPreferences) : Payment =
            Payment(
                preferences.getInt(COURTS_KEY, 1),
                preferences.getInt(PLAYERS_KEY, 1),
                preferences.getFloat(PRICE_PER_UNIT_KEY, DEFAULT_PRICE_PER_UNIT).toDouble()
            )
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_settings)
        buttonOK.setOnClickListener{ onSave() }
        val preferences = this.applicationContext.getSharedPreferences(
            PREFERENCE_FILENAME, Context.MODE_PRIVATE)
        val payment = defaultPayment(preferences)
        this.editPricePerUnit.setText("${payment.pricePerUnit}")
        this.editDefaultPlayers.setText("${payment.players}")
    }

    private fun onSave(){
        val decimalFormat = DecimalFormat.getInstance(Locale.getDefault())
        val pricePerUnit = decimalFormat.parse(this.editPricePerUnit.text.toString()).toFloat()
        val players = decimalFormat.parse(this.editDefaultPlayers.text.toString()).toInt()
        val preferences = this.applicationContext.getSharedPreferences(
            PREFERENCE_FILENAME, Context.MODE_PRIVATE).edit()
        preferences.putFloat(PRICE_PER_UNIT_KEY, pricePerUnit)
        preferences.putInt(PLAYERS_KEY, players)
        preferences.commit()
        setResult(Activity.RESULT_OK)

        finish()
    }
}
