package at.htl.boehm.meettheteacher.fragments


import android.os.Bundle
import android.os.Handler
import android.support.v4.app.Fragment
import android.support.v7.widget.LinearLayoutManager
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import at.htl.boehm.meettheteacher.DbWorkerThread
import at.htl.boehm.meettheteacher.R
import at.htl.boehm.meettheteacher.adapter.TeachersAdapter
import at.htl.boehm.meettheteacher.db.entities.Teacher
import at.htl.boehm.meettheteacher.db.TeacherDatabase
import kotlinx.android.synthetic.main.fragment_display_teachers.*

class DisplayTeachersFragment : Fragment() {
    internal val LOG_TAG: String = DisplayTeachersFragment::class.java.simpleName

    private var mDb: TeacherDatabase? = null

    private val mUiHandler = Handler()

    private lateinit var mDbWorkerThread: DbWorkerThread

    val teachers: ArrayList<Teacher> = ArrayList()

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?,
                              savedInstanceState: Bundle?): View? {
        mDbWorkerThread = DbWorkerThread("dbWorkerThread")
        mDbWorkerThread.start()

        mDb = TeacherDatabase.getInstance(context)
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_display_teachers, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        rv_teachers.setHasFixedSize(true)
        rv_teachers.layoutManager = LinearLayoutManager(context)

        getTeachersFromDb()

        setTeachersAdapter()
    }

    private fun getTeachersFromDb() {
        val task = Runnable {
            val teachersData =
                    mDb?.teacherDao()?.getAllTeachers()
            if(teachersData != null){
                teachersData.forEach {
                    teachers.add(it)
                }
                teachers.sortBy { it.name }
                mUiHandler.post(Runnable { setTeachersAdapter() })
            }
        }
        mDbWorkerThread.postTask(task)
    }

    private fun setTeachersAdapter(){
        Log.e(LOG_TAG, "INSIDE ADAPTER")
        rv_teachers.adapter = TeachersAdapter(teachers)
    }
}
