package at.htl.boehm.meettheteacher.fragments


import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.os.Bundle
import android.support.v4.app.Fragment
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import at.htl.boehm.meettheteacher.DbWorkerThread
import at.htl.boehm.meettheteacher.asyncTasks.LoadTeachersAsyncTask
import at.htl.boehm.meettheteacher.asyncTasks.LoadTeachersImageAsyncTask
import at.htl.boehm.meettheteacher.R
import at.htl.boehm.meettheteacher.db.entities.Teacher
import at.htl.boehm.meettheteacher.db.TeacherDatabase
import kotlinx.android.synthetic.main.fragment_download_teachers.*
import java.io.ByteArrayOutputStream

class DownloadTeachersFragment() : Fragment() {
    internal val LOG_TAG = DownloadTeachersFragment::class.java.simpleName

    val url = "https://www.htl-leonding.at/kontakt/lehrer-innen/"

    private var mDb: TeacherDatabase? = null

    private lateinit var mDbWorkerThread: DbWorkerThread

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?,
                              savedInstanceState: Bundle?): View? {
        mDbWorkerThread = DbWorkerThread("dbWorkerThread")
        mDbWorkerThread.start()

        mDb = TeacherDatabase.getInstance(context)
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_download_teachers, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        loadTeachersFromHomepage()
    }

    private fun loadTeachersFromHomepage() {
        deleteAllTeachers()

        val loadTeachersAsyncTask = LoadTeachersAsyncTask {
            pb_progress.max = it.size
            it.forEach(::loadTeachersBitmap)
        }
        try{
            loadTeachersAsyncTask.execute(url)
            Log.e(LOG_TAG, "asynctask started")
        }catch(e: Exception){
            e.printStackTrace()
            Log.e(LOG_TAG, "Loading Teachers Failed ${e.message}")
        }
    }

    private fun loadTeachersBitmap(teacher: Teacher) {
        if(teacher.detailsId == 0) {
            teacher.bitmap = toByteArray(BitmapFactory.decodeResource(resources, R.drawable.anonym))
            pb_progress.progress++
            insertTeacherInDb(teacher)
        }
        else{
            val loadTeachersImageAsyncTask = LoadTeachersImageAsyncTask {
                tv_act_teacher.text = it.name
                insertTeacherInDb(teacher)
                pb_progress.progress++

                if (pb_progress.progress >= pb_progress.max) {
                    val displayTeachersFragment = DisplayTeachersFragment()
                    activity?.supportFragmentManager?.beginTransaction()?.replace(R.id.fl_main, displayTeachersFragment)?.commit()
                }
            }
            try{
                loadTeachersImageAsyncTask.execute(teacher)
            }catch (e: Exception){
                e.printStackTrace()
                Log.e(LOG_TAG, "getImage() load image, exception: ${e.message}")
            }
        }
    }

    private fun insertTeacherInDb(teacher: Teacher){
        val task = Runnable { mDb?.teacherDao()?.insertTeacher(teacher) }
        mDbWorkerThread.postTask(task)
    }

    private fun deleteAllTeachers() {
        val task = Runnable { mDb?.teacherDao()?.deleteAll() }
        mDbWorkerThread.postTask(task)
    }

    private fun toByteArray(bitmap: Bitmap?): ByteArray {
        val stream = ByteArrayOutputStream()
        bitmap?.compress(Bitmap.CompressFormat.PNG, 0, stream)
        return stream.toByteArray()
    }
}
