package at.htlleonding.room.notekeeper.model

import android.app.Application
import android.arch.lifecycle.AndroidViewModel
import android.arch.lifecycle.LiveData
import android.util.Log
import at.htlleonding.room.notekeeper.db.NotesDao
import at.htlleonding.room.notekeeper.db.NotesRoomDatabase


class NoteViewModel(application: Application) : AndroidViewModel(application) {
    private val notesDao: NotesDao
    val notes: LiveData<List<Note>>


    companion object{
        val LOG_TAG = NoteViewModel::class.java.simpleName
    }

    init{
        notesDao = NotesRoomDatabase.getInstance(application)!!.notesDao()
        notes = notesDao.allNotes
    }


    fun add(note: Note) {
        performDbTask { notesDao.insert(note)}
    }

    fun delete(note: Note){
        performDbTask{notesDao.delete(note)}
    }

    fun update(note: Note) {
        performDbTask{notesDao.update(note)}

    }

    private fun performDbTask(dbTask: () -> Unit?){
        Log.d(
            LOG_TAG, "Accessing DB in thread ${Thread.currentThread().name}")
        dbTask()
    }

}