package at.htlleonding.room.notekeeper.model

import android.app.Application
import android.arch.lifecycle.AndroidViewModel
import android.arch.lifecycle.LiveData
import android.os.AsyncTask
import android.util.Log
import at.htlleonding.room.notekeeper.db.NotesDao
import at.htlleonding.room.notekeeper.db.NotesRoomDatabase
import java.lang.Runnable


class NoteViewModel(application: Application) : AndroidViewModel(application) {
    private val notesDao: NotesDao
    val notes: LiveData<List<Note>>


    companion object{
        val LOG_TAG = NoteViewModel::class.java.simpleName
    }

    init{
        notesDao = NotesRoomDatabase.getInstance(application)!!.notesDao()
        notes = notesDao.allNotes
    }


    fun add(note: Note) {
        doAsync { notesDao.insert(note)}
    }

    fun delete(note: Note){
        doAsync{notesDao.delete(note)}
    }

    fun update(note: Note) {
        doAsync{notesDao.update(note)}

    }

    private fun doAsync(callback: () -> Unit?){
        DbAsyncTask().execute(callback)
    }

    class DbAsyncTask: AsyncTask<() -> Unit?, Void, Void>(){
        override fun doInBackground(vararg params: (() -> Unit?)): Void? {
            Log.d(LOG_TAG, "Accessing DB in thread ${Thread.currentThread().name} ")
            params[0]()
            return null
        }
    }

}