package at.htlleonding.room.notekeeper.model

import android.app.Application
import android.arch.lifecycle.AndroidViewModel
import android.arch.lifecycle.LiveData
import android.util.Log
import at.htlleonding.room.notekeeper.db.NotesDao
import at.htlleonding.room.notekeeper.db.NotesRoomDatabase

import kotlin.concurrent.thread


class NoteViewModel(application: Application)
        : AndroidViewModel(application) {
    private val notesDao: NotesDao
    val notes: LiveData<List<Note>>

    init{
        notesDao = NotesRoomDatabase.getInstance(application).notesDao()
        notes = notesDao.allNotes
    }


    companion object{
        val LOG_TAG = NoteViewModel::class.java.simpleName
    }

    fun add(note: Note) {
        Log.d(LOG_TAG, "Calling Insert in thread ${Thread.currentThread().name}")
        thread{
            Log.d(
                LOG_TAG, "Inserting in thread ${Thread.currentThread().name}")
            notesDao.insert(note)
        }
    }

    fun delete(note: Note){
        thread{
            Log.d(
                LOG_TAG, "Deleting in thread ${Thread.currentThread().name}")
            notesDao.delete(note)
        }
    }

    fun update(note: Note) {
        thread{
            Log.d(
                LOG_TAG, "Updating in thread ${Thread.currentThread().name}")
            notesDao.update(note)
        }
    }
}