package at.htlleonding.room.notekeeper

import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.widget.Toast
import at.htlleonding.room.notekeeper.model.Note
import at.htlleonding.room.notekeeper.model.NoteViewModel

import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.content_main.*

class MainActivity : AppCompatActivity() {

    companion object{
        val NEW_NOTE_REQUEST = 1
        val UPDATE_NOTE_REQUEST = 2
    }

    private lateinit var noteViewModel: NoteViewModel

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        setSupportActionBar(toolbar)
        noteViewModel = ViewModelProviders.of(this).get(NoteViewModel::class.java)
        val noteListAdapter = NoteListAdapter()
        recyclerview.adapter = noteListAdapter
        recyclerview.layoutManager = LinearLayoutManager(this)

        fab.setOnClickListener { view ->
            val intent = Intent(this, NewNoteActivity::class.java)
            startActivityForResult(intent, NEW_NOTE_REQUEST )
        }

        noteViewModel.notes.observe(this, Observer { notes ->
            notes?.let {
                noteListAdapter.setNotes(notes)
            }
        })
        noteListAdapter.deleteCallBack = {noteViewModel.delete(it)}
        noteListAdapter.updateCallBack = {
            val intent = Intent(this, UpdateNoteActivity::class.java)
            intent.putExtra(UpdateNoteActivity.NOTE_ID, it.id)
            intent.putExtra(UpdateNoteActivity.NOTE_TEXT, it.note)
            startActivityForResult(intent, UPDATE_NOTE_REQUEST )
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        val message = when {
            requestCode == NEW_NOTE_REQUEST && resultCode == RESULT_OK -> {
                val note = Note(null, data!!.getStringExtra(NewNoteActivity.NOTE_ADDED))
                noteViewModel.add(note)
                "Note added: " + note.note + " id = " + note.id
            }
            requestCode == UPDATE_NOTE_REQUEST && resultCode == RESULT_OK -> {
                val id = data?.getLongExtra(UpdateNoteActivity.NOTE_ID, -1)
                val updated = data?.getStringExtra(UpdateNoteActivity.NOTE_TEXT)
                noteViewModel.update(Note(id, updated!!))
                "Note update: " + updated
            }
            else ->  "Nothing saved"
        }
        Toast.makeText(
            applicationContext,
            message,
            Toast.LENGTH_LONG
        ).show()
    }
}
