package at.htlleonding.room.notekeeper.model

import android.app.Application
import android.arch.lifecycle.AndroidViewModel
import android.arch.lifecycle.LiveData
import android.util.Log
import at.htlleonding.room.notekeeper.db.DbWorkerThread
import at.htlleonding.room.notekeeper.db.NotesDao
import at.htlleonding.room.notekeeper.db.NotesRoomDatabase
import kotlinx.coroutines.*
import java.lang.Runnable
import kotlin.concurrent.thread


class NoteViewModel(application: Application) : AndroidViewModel(application) {
    private val notesDao: NotesDao
    val notes: LiveData<List<Note>>

    private val workerThread: DbWorkerThread

    companion object{
        val LOG_TAG = NoteViewModel::class.java.simpleName
    }

    init{
        workerThread = DbWorkerThread("NotesDBWorkerThread")
        workerThread.start()
        notesDao = NotesRoomDatabase.getInstance(application)!!.notesDao()
        notes = notesDao.allNotes
    }


    fun add(note: Note) {
        doAsync { notesDao.insert(note)}
    }

    fun delete(note: Note){
        doAsync{notesDao.delete(note)}
    }

    fun update(note: Note) {
        doAsync{notesDao.update(note)}

    }

    private fun doAsync(callback: () -> Unit?){
        workerThread.postTask(Runnable({
            callback()
            Log.d(LOG_TAG, "Accessing DB in thread ${Thread.currentThread().name} ")
        }))
    }

}