package at.htlleonding.room.notekeeper.model

import android.app.Application
import android.arch.lifecycle.AndroidViewModel
import android.arch.lifecycle.LiveData
import android.util.Log
import at.htlleonding.room.notekeeper.db.NotesDao
import at.htlleonding.room.notekeeper.db.NotesRoomDatabase
import kotlinx.coroutines.*


class NoteViewModel(application: Application)
        : AndroidViewModel(application) {
    private val notesDao: NotesDao
    val notes: LiveData<List<Note>>
    init{
        notesDao = NotesRoomDatabase.getInstance(application)!!.notesDao()
        notes = notesDao.allNotes
    }

    companion object{
        val LOG_TAG = NoteViewModel::class.java.simpleName
    }

    private val viewModelJob = Job()
    private val uiScope = CoroutineScope(Dispatchers.Main + viewModelJob)

    override fun onCleared() {
        super.onCleared()
        viewModelJob.cancel()
    }

    fun add(note: Note) {
        uiScope.launch {
            doAsync { notesDao.insert(note) }
        }
    }

    fun delete(note: Note){
        uiScope.launch {
            doAsync({notesDao.delete(note)})
        }
    }

    fun update(note: Note) {
        uiScope.launch {
            doAsync{ notesDao.update(note) }
        }
    }

    private suspend fun doAsync(dbTask: () -> Unit?){
        Log.d(LOG_TAG,
            "Calling DB in thread ${Thread.currentThread().name}," +
                    " scope: ${this.toString()}")
        withContext(Dispatchers.IO){
            Log.d(
                LOG_TAG, "Accessing DB in thread ${Thread.currentThread().name}," +
                        " scope: ${this.toString()} ${this.coroutineContext.toString()}")
            dbTask()
        }
    }

}