package at.htl.paging.repository

import android.util.Log
import androidx.paging.PageKeyedDataSource
import at.htl.paging.api.GitRepo
import at.htl.paging.api.GitRepoResponse
import at.htl.paging.api.GitRepoServiceBuilder
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class GitRepoDataSource : PageKeyedDataSource<Int, GitRepo>(){
    companion object{
        val LOG_TAG = GitRepoDataSource::class.java.simpleName
        val TOPIC = "android"
        val FIRST_PAGE = 1
        val PAGE_SIZE = 5
    }
    override fun loadInitial(params: LoadInitialParams<Int>, callback: LoadInitialCallback<Int, GitRepo>) {
        val service = GitRepoServiceBuilder.getGitRepoService()
        val call = service.getRepositories(FIRST_PAGE, PAGE_SIZE, TOPIC)

        call.enqueue(object: Callback<GitRepoResponse> {

            override fun onResponse(call: Call<GitRepoResponse>, response: Response<GitRepoResponse>) {

                if (response.isSuccessful) {
                    val apiResponse = response.body()!!
                    val responseItems = apiResponse.items

                    responseItems?.let {
                        Log.d(LOG_TAG, "Load Initial Response - next Page: ${FIRST_PAGE + 1}")
                        callback.onResult(it, null, FIRST_PAGE + 1)
                    }


                }
            }

            override fun onFailure(call: Call<GitRepoResponse>, t: Throwable) {
            }
        })
    }

    override fun loadAfter(params: LoadParams<Int>, callback: LoadCallback<Int, GitRepo>) {
        val service = GitRepoServiceBuilder.getGitRepoService()
        val call = service.getRepositories(params.key, PAGE_SIZE, TOPIC)

        call.enqueue(object: Callback<GitRepoResponse> {

            override fun onResponse(call: Call<GitRepoResponse>, response: Response<GitRepoResponse>) {

                if (response.isSuccessful) {
                    val apiResponse = response.body()!!
                    val responseItems = apiResponse.items
                    val nextPage = if (params.key < apiResponse.totalCount) {
                        params.key + 1
                    }
                    else {
                        apiResponse.totalCount
                    }
                    responseItems?.let {
                        Log.d(LOG_TAG, "Load After Response - nextPage = ${nextPage} size = ${it.size}")
                        callback.onResult(it, nextPage )
                    }


                }
            }

            override fun onFailure(call: Call<GitRepoResponse>, t: Throwable) {
            }
        })
    }

    override fun loadBefore(params: LoadParams<Int>, callback: LoadCallback<Int, GitRepo>) {
        val service = GitRepoServiceBuilder.getGitRepoService()
        val call = service.getRepositories(params.key, PAGE_SIZE, TOPIC)

        call.enqueue(object: Callback<GitRepoResponse> {

            override fun onResponse(call: Call<GitRepoResponse>, response: Response<GitRepoResponse>) {

                if (response.isSuccessful) {
                    val apiResponse = response.body()!!
                    val responseItems = apiResponse.items
                    val nextPage = if (params.key > 1){
                        params.key - 1
                    }
                    else{
                        0
                    }

                    responseItems?.let {
                        callback.onResult(it, nextPage )
                        Log.d(LOG_TAG, "Load After Response - nextPage = ${nextPage} size = ${it.size}")

                    }


                }
            }

            override fun onFailure(call: Call<GitRepoResponse>, t: Throwable) {
            }
        })
    }

}