package at.htl.leonding.swimresults.swimmerlist

import android.app.Application
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.LiveData
import at.htl.leonding.swimresults.database.SwimResultsDao
import at.htl.leonding.swimresults.database.SwimResultsDatabase
import at.htl.leonding.swimresults.database.Swimmer
import kotlinx.coroutines.*

class SwimmerListViewModel(application: Application) : AndroidViewModel(application) {

    private val database: SwimResultsDao
    private val viewModelJob = Job()
    private val uiScope = CoroutineScope(Dispatchers.Main + viewModelJob)

    val swimmers: LiveData<List<Swimmer>>
    init{
        database = SwimResultsDatabase.getInstance(application)
                .swimResultsDao()
        swimmers = database.getAllSwimmers()
    }

    fun delete(swimmer: Swimmer) {
        uiScope.launch{
            withContext(Dispatchers.IO){
                database.deleteSwimmer(swimmer)
            }
        }
    }

    override fun onCleared() {
        super.onCleared()
        viewModelJob.cancel()
    }
}

